#include <iostream>
#include <cpptest.h>
#include <fstream>
#include "ecosystem.h"
#include "item_type.h"
#include "scores.h"
#include "damage_types.h"
#include "range_weapon_type.h"
#include "roles.h"
using namespace std;
static void usage()
{
	cout << "usage: mytest [MODE]\n"
		 << "where MODE may be one of:\n"
		 << "  --compiler\n"
		 << "  --html\n"
		 << "  --text-terse (default)\n"
		 << "  --text-verbose\n";
	exit(0);
}
static auto_ptr<Test::Output> cmdline(int argc, char* argv[])
{
	if (argc > 2)
		usage(); // will not return
	
	Test::Output* output = 0;
	
	if (argc == 1)
		output = new Test::TextOutput(Test::TextOutput::Verbose);
	else
	{
		const char* arg = argv[1];
		if (strcmp(arg, "--compiler") == 0)
		{
#ifdef _MSC_VER
			output = new Test::CompilerOutput(Test::CompilerOutput::MSVC, msvDebug);
#elif defined(__GNUC__)
			output = new Test::CompilerOutput(Test::CompilerOutput::GCC);
#else
			output = new Test::CompilerOutput;
#endif
		}
		else if (strcmp(arg, "--html") == 0)
			output =  new Test::HtmlOutput;
		else if (strcmp(arg, "--text-terse") == 0)
			output = new Test::TextOutput(Test::TextOutput::Terse);
		else if (strcmp(arg, "--text-verbose") == 0)
			output = new Test::TextOutput(Test::TextOutput::Verbose);
		else
		{
			cout << "invalid commandline argument: " << arg << endl;
			usage(); // will not return
		}
	}

	return auto_ptr<Test::Output>(output);
}
int main (int argc, char* argv[]){
  static const char *outputFileName = "result.html";
  Test::Suite ts;
  ts.add(auto_ptr<Test::Suite>(new ECOSystemTest));
  ts.add(auto_ptr<Test::Suite>(new TItemTypeTest));
  ts.add(auto_ptr<Test::Suite>(new ScoreTest));
  ts.add(auto_ptr<Test::Suite>(new EDamageTypeTest));
  ts.add(auto_ptr<Test::Suite>(new RolesTest));
  auto_ptr<Test::Output> output(cmdline(argc, argv));
  ts.run(*output);
  Test::HtmlOutput* const html = dynamic_cast<Test::HtmlOutput*>(output.get());
		if (html)
		{
			std::ofstream fout(outputFileName);
			html->generate(fout, true, "NeLTest");
		}
  return 0;
}
